import pygame, math
import var, text

def makegrey(img):
  img2 = img.copy()
  for x in xrange(0, img.get_width()):
    for y in xrange(0, img.get_height()):
      r, g, b, alf = img.get_at((x, y))
      img2.set_at((x, y), (r/3, g/3, b/3, alf))
  return img2

def initialize():
  global mimg, fimg, eimg, simg, wimg, limg, rimg, imgs
  global mimg0, fimg0, eimg0, simg0, wimg0, limg0
  global emap, fmap, mmap, fspot
  global slist, rlist

  mimg = pygame.image.load("mine.png").convert_alpha()
  mimg0 = makegrey(mimg)
  fimg = pygame.image.load("factory.png").convert_alpha()
  fimg0 = makegrey(fimg)
  eimg = pygame.image.load("electric.png").convert_alpha()
  eimg0 = makegrey(eimg)
  simg = pygame.image.load("shield.png").convert_alpha()
  simg0 = makegrey(simg)
  wimg = pygame.image.load("warhead.png").convert_alpha()
  wimg0 = makegrey(wimg)
  limg = pygame.image.load("launch.png").convert_alpha()
  limg0 = makegrey(limg)

  rimg = pygame.image.load("bulldozer.png").convert_alpha()
  liftoff = pygame.image.load("liftoff.png").convert_alpha()


  imgs = (fimg, eimg, mimg, simg, limg, wimg, rimg)

  emap = pygame.Surface(var.msize).convert_alpha()
  fmap = pygame.Surface(var.msize).convert_alpha()
  mmap = pygame.Surface(var.msize).convert_alpha()

  fspot = pygame.Surface((2*var.srad, 2*var.srad)).convert_alpha()
  fspot.fill(var.black)
  pygame.draw.circle(fspot, var.thinblue, (var.srad, var.srad), var.srad)

  slist = []
  slist.append(Factory((var.xm2 + 60, var.ym2), True))
  slist.append(Electric((var.xm2 - 60, var.ym2), True))
  rlist = set()
  toupdate = True

def update():
  global slist, rlist, toupdate, emap, fmap, mmap, fspot, rlist
  if not toupdate: return
  if len(rlist):
    rlist = list(rlist)
    rlist.sort()
    rlist.reverse()
    for r in rlist:
      slist.pop(r)
    rlist = set()
  while toupdate:
    toupdate = False
    for s in slist:
      s.updatefunctioning(slist)
  for s in slist:
    s.updatebuildspeed(slist)

  emap.fill(var.black)
  fmap.fill(var.black)
  mmap.fill(var.black)
  for s in slist:
    if s.isfactory():
      for p in s.spot:
        fmap.blit(fspot, p)
    elif s.iselectric():
      for x,y in s.spot:
        pygame.draw.circle(emap, var.thinyellow, (x+var.srad, y+var.srad), var.srad)
    elif s.takesspace():
      for x,y in s.spot:
        pygame.draw.circle(mmap, var.thinred, (x+var.srad, y+var.srad), var.srad)

def remove(x, y, r):
  global slist, rlist, toupdate
  for j in xrange(0, len(slist)):
    xs, ys = slist[j].pos
    dx = (xs - x + var.xm2) % var.xm - var.xm2
    dy = (ys - y + var.ym2) % var.ym - var.ym2
    if dx*dx + dy*dy < r*r:
      rlist.add(j)
      toupdate = True

def toonear(x, y):
  global slist
  for j in xrange(0, len(slist)):
    xs, ys = slist[j].pos
    dx = (xs - x + var.xm2) % var.xm - var.xm2
    dy = (ys - y + var.ym2) % var.ym - var.ym2
    if dx*dx + dy*dy < 32*32:
      return True
  return False

def infect(imap):
  global slist, rlist, toupdate
  for j in xrange(0, len(slist)):
    xs, ys = slist[j].pos
    if imap.get_at((xs % var.xm, ys % var.ym)) == var.grey:
      rlist.add(j)
      toupdate = True
def disinfect(amap):
  global slist
  amap.fill(var.white)
  for s in slist:
    if s.isshield():
      for x,y in s.spot:
        nx = (x + var.srad) / var.cell
        ny = (y + var.srad) / var.cell
        nr = s.r / var.cell
        pygame.draw.circle(amap, var.black, (nx, ny), nr, 1)

class Structure:
  def __init__(self, pos, img0, img1, btime):
    global toupdate
    self.pos = pos
    self.buildingimg = img0
    self.img = img1
    self.xa, self.ya = pos
    self.buildtime = btime
    self.buildspeed = 0
    self.lastdraw = pygame.time.get_ticks()
    if btime > 0:
      self.buildfrac = 0
      self.finished = False
    else:
      self.buildfrac = 1
      self.finished = True
    toupdate = True
    self.xr = img1.get_width() / 2
    self.yr = img1.get_height() / 2
    self.ir = int(self.xr * 1.7)
    self.functioning = False
    self.spot = set()
    a0 = int(math.floor(float(self.xa - var.srad) / var.xm))
    a1 = int(math.floor(float(self.xa + var.srad) / var.xm))
    b0 = int(math.floor(float(self.ya - var.srad) / var.ym))
    b1 = int(math.floor(float(self.ya + var.srad) / var.ym))
    for a in xrange(a0, a1+1):
      for b in xrange(b0, b1+1):
        x = self.xa - a * var.xm - var.srad
        y = self.ya - b * var.ym - var.srad
        self.spot.add((x,y))
    self.functiontime = 0

  def place(self, pv, pf, pm):
    xv, yv = pv
    xf, yf = pf
    xm, ym = pm
    xp, yp = self.pos
    self.xa = (xp + xv/2 - xf + (xm - xv)/2) % xm - (xm - xv)/2
    self.ya = (yp + yv/2 - yf + ym/2) % ym - ym/2
  def d2(self, other):
    dx = (self.pos[0] - other.pos[0] + var.xm2) % var.xm - var.xm2
    dy = (self.pos[1] - other.pos[1] + var.ym2) % var.ym - var.ym2
    return dx*dx + dy*dy

  def updatebuildspeed(self, others):
    if self.finished: return
    self.buildspeed = 0
    for o in others:
      if o.isfactory():
        if self.d2(o) < o.radius * o.radius:
          self.buildspeed += 1
  def updatefunctioning(self, others):
    global toupdate
    if not self.finished: return
    wasfunctioning = self.functioning
    self.functioning = False
    for o in others:
      if o.iselectric() and self.d2(o) < var.srad * var.srad:
        self.functioning = True
    if wasfunctioning != self.functioning: toupdate = True

  def draw0(self, surf):
    pass
  def draw1(self, surf):
    global toupdate
    now = pygame.time.get_ticks()
    if not self.finished:
      self.buildfrac += self.buildspeed * float(now - self.lastdraw) / self.buildtime
      if self.buildfrac < 1:
        cr = pygame.draw.circle(surf, var.red, (self.xa, self.ya), self.ir)
        try:
          pygame.draw.arc(surf, var.green, cr, 1.571 - 6.283 * self.buildfrac, 1.571, self.ir-4)
          pygame.draw.arc(surf, var.green, cr, 1.53 - 6.283 * self.buildfrac, 1.571, self.ir-4)
          pygame.draw.circle(surf, var.black, (self.xa, self.ya), self.ir, 1)
        except:
          pass
        surf.blit(self.buildingimg, (self.xa - self.xr, self.ya - self.yr))
      else:
        self.finished = True
        self.functioning = self.iselectric()
        toupdate = True
      self.lastdraw = now
    if self.finished:
      if self.functioning:
        surf.blit(self.img, (self.xa - self.xr, self.ya - self.yr))
        self.count(now - self.lastdraw)
        self.lastdraw = now
      else:
        surf.blit(self.buildingimg, (self.xa - self.xr, self.ya - self.yr))
  def count(self, dt):
    pass
    
  def isfactory(self):
    return False
  def iselectric(self):
    return False
  def takesspace(self):
    return True
  def isshield(self):
    return False

class Shield (Structure):
  def __init__(self, pos):
    Structure.__init__(self, pos, simg0, simg, 1000 * var.btime['shield'])
    self.r = var.srad/2
    self.time0 = pygame.time.get_ticks()

  def draw0(self, surf):
    if not self.functioning: return
    theta = (pygame.time.get_ticks() - self.time0) / 400.
    pygame.draw.circle(surf, var.red, (self.xa, self.ya), self.r, 3)
    dx = int(math.floor(self.r * math.cos(theta) + 0.5))
    dy = int(math.floor(self.r * math.sin(theta) + 0.5))
    pygame.draw.line(surf, var.red, (self.xa + dx, self.ya + dy), (self.xa - dx, self.ya - dy), 2)
  def isshield(self):
    return self.functioning

class Mine (Structure):
  def __init__(self, pos, finished = False):
    if finished:
      Structure.__init__(self, pos, mimg0, mimg, 0)
    else:
      Structure.__init__(self, pos, mimg0, mimg, 1000*var.btime['mine'])
    self.clock = 0
  def count(self, dt):
    self.clock += dt
    while self.clock > 1000:
      self.clock -= 1000
      var.money += 0.1
class Electric (Structure):
  def __init__(self, pos, finished = False):
    if finished:
      Structure.__init__(self, pos, eimg0, eimg, 0)
      self.functioning = True
    else:
      Structure.__init__(self, pos, eimg0, eimg, 1000*var.btime['electric'])
    self.radius = 160
  def iselectric(self):
    return self.functioning
  def updatefunctioning(self, others):
    global toupdate
    if self.functioning or not self.finished: return
    self.functioning = True
    toupdate = True
  def takesspace(self):
    return False
    
class Factory (Structure):
  def __init__(self, pos, finished = False):
    if finished:
      Structure.__init__(self, pos, fimg0, fimg, 0)
    else:
      Structure.__init__(self, pos, fimg0, fimg, 1000*var.btime['factory'])
    self.radius = 160
  def isfactory(self):
    return self.functioning
  def takesspace(self):
    return False

class Launch (Structure):
  def __init__(self, pos):
    Structure.__init__(self, pos, limg0, limg, 1000*var.btime['launch'])
    self.clock = 19999
  def count(self, dt):
    global toupdate
    self.clock -= dt
    if self.clock < 1000:
      self.functioning = False
      toupdate = True
      var.launching = True
  def draw1(self, surf):
    Structure.draw1(self, surf)
    if self.finished:
      ltext = text.smallfont.render(str(int(self.clock / 1000)), True, var.black)  
      surf.blit(ltext, (self.xa - ltext.get_width()/2, self.ya))
    
class Warhead (Structure):
  def __init__(self, pos):
    Structure.__init__(self, pos, limg0, limg, 1000*var.btime['warhead'])



